<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use SimpleXMLIterator;

class YoutubeParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/youtube';

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new Item($element);
        $item->fill([
            'type' => $item->getType(),
            'uri' => $this->createEmbedUrl((string)$item->getItemAttribute('uri')),
            'thumb' => $this->getThumbData($element),
            'start' => $this->getStartTime($item),
            'embedId' => $this->createEmbedId($item)
        ]);

        return [$item];
    }

    private function getStartTime(Item $item): string
    {
        $data = $item->getFirstChild('data');

        return $data instanceof Item ? $data->get('start', '0') : '0';
    }

    private function getThumbData(SimpleXMLIterator $el): array
    {
        $links = (new LinkParser())->parseLinks($el);
        foreach ($links as $link) {
            if ($link instanceof Item && $link->getType() === 'image/jpg') {
                return $this->parseThumb($link);
            }
        }

        return [];
    }

    private function createEmbedUrl(string $url): string
    {
        return str_replace('watch?v=', 'embed/', $url);
    }

    private function createEmbedId(Item $item): string
    {
        $url = $item->get('url', '');

        if (empty($url)) {
            return '';
        }

        $parsedUrl = parse_url($url);

        switch ($parsedUrl['host']) {
            case 'youtu.be':
                return ltrim($parsedUrl['path'] ?? '', '/');
            case 'www.youtube.com':
                parse_str($parsedUrl['query'] ?? '', $parameters);

                return $parameters['v'] ?? '';
            default:
                return '';
        }
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }

    private function parseThumb(Item $link): array
    {
        $data = $link->getFirstChild('data');

        return array_replace([
            'width' => 0,
            'height' => 0,
            'url' => $link->getItemAttribute('url')
        ], $data instanceof Item ? $data->getProperties() : []);
    }
}
